@@ -42,19 +42,30 @@
 
 #include "precomp.hpp"
 
+#ifndef _DEBUG
+#define QT_NO_DEBUG_OUTPUT
+#endif
+
 #if defined( HAVE_QT_OPENGL )
 #include <QtOpenGL>
-#include <QGLWidget>
+
+  // QGLWidget deprecated and no longer functions with Qt6, use QOpenGLWidget instead
+  #ifdef HAVE_QT6
+  #include <QOpenGLWidget>
+  #else
+  #include <QGLWidget>
+  #endif
+
 #endif
 
 #include <QAbstractEventDispatcher>
-#include <QtGui/QApplication>
+#include <QApplication>
 #include <QFile>
 #include <QPushButton>
-#include <QtGui/QGraphicsView>
+#include <QGraphicsView>
 #include <QSizePolicy>
 #include <QInputDialog>
-#include <QtGui/QBoxLayout>
+#include <QBoxLayout>
 #include <QSettings>
 #include <qtimer.h>
 #include <QtConcurrentRun>
@@ -72,13 +83,14 @@
 #include <QDate>
 #include <QFileDialog>
 #include <QToolBar>
+#include <QClipboard>
+
 #include <QAction>
-#include <QPushButton>
 #include <QCheckBox>
 #include <QRadioButton>
 #include <QButtonGroup>
 #include <QMenu>
-#include <QtTest/QTest>
+#include <QTest>
 
 //start private enum
 enum { CV_MODE_NORMAL = 0, CV_MODE_OPENGL = 1 };
@@ -87,6 +99,7 @@ enum { CV_MODE_NORMAL = 0, CV_MODE_OPENGL = 1 };
 enum {	shortcut_zoom_normal 	= Qt::CTRL + Qt::Key_Z,
         shortcut_zoom_imgRegion = Qt::CTRL + Qt::Key_X,
         shortcut_save_img		= Qt::CTRL + Qt::Key_S,
+        shortcut_copy_clipbrd   = Qt::CTRL + Qt::Key_C,
         shortcut_properties_win	= Qt::CTRL + Qt::Key_P,
         shortcut_zoom_in 		= Qt::CTRL + Qt::Key_Plus,//QKeySequence(QKeySequence::ZoomIn),
         shortcut_zoom_out		= Qt::CTRL + Qt::Key_Minus,//QKeySequence(QKeySequence::ZoomOut),
@@ -95,7 +108,6 @@ enum {	shortcut_zoom_normal 	= Qt::CTRL + Qt::Key_Z,
         shortcut_panning_up 	= Qt::CTRL + Qt::Key_Up,
         shortcut_panning_down 	= Qt::CTRL + Qt::Key_Down
     };
-
 //end enum
 
 class CvWindow;
@@ -129,9 +141,12 @@ public slots:
     void displayStatusBar( QString name, QString text, int delayms );
     void timeOut();
     void toggleFullScreen(QString name, double flags );
+    CvRect getWindowRect(QString name);
     double isFullScreen(QString name);
     double getPropWindow(QString name);
     void setPropWindow(QString name, double flags );
+    void setWindowTitle(QString name, QString title);
+    double getWindowVisible(QString name);
     double getRatioWindow(QString name);
     void setRatioWindow(QString name, double arg2 );
     void saveWindowParameters(QString name);
@@ -248,11 +263,11 @@ private:
     QPointer<QPushButton > label;
     CvTrackbarCallback callback;
     CvTrackbarCallback2 callback2;//look like it is use by python binding
-    int* dataSlider;
+    int* dataSlider;  // deprecated
     void* userdata;
 };
 
-//Both are top level window, so that a way to differenciate them.
+//Both are top level window, so that a way to differentiate them.
 //if (obj->metaObject ()->className () == "CvWindow") does not give me robust result
 
 enum typeWindow { type_CvWindow = 1, type_CvWinProperties = 2 };
@@ -272,9 +287,9 @@ public:
     QPointer<QBoxLayout> myLayout;
 
 private:
-    void closeEvent ( QCloseEvent * e );
-    void showEvent ( QShowEvent * event ) ;
-    void hideEvent ( QHideEvent * event ) ;
+    void closeEvent ( QCloseEvent * e ) CV_OVERRIDE;
+    void showEvent ( QShowEvent * event ) CV_OVERRIDE;
+    void hideEvent ( QHideEvent * event ) CV_OVERRIDE;
 };
 
 
@@ -283,7 +298,6 @@ class CvWindow : public CvWinModel
     Q_OBJECT
 public:
     CvWindow(QString arg2, int flag = CV_WINDOW_NORMAL);
-    ~CvWindow();
 
     void setMouseCallBack(CvMouseCallback m, void* param);
 
@@ -293,6 +307,7 @@ public:
     double getRatio();
     void setRatio(int flags);
 
+    CvRect getWindowRect();
     int getPropWindow();
     void setPropWindow(int flags);
 
@@ -332,7 +347,8 @@ public:
     QPointer<QLabel> myStatusBar_msg;
 
 protected:
-    virtual void keyPressEvent(QKeyEvent* event);
+    virtual void keyPressEvent(QKeyEvent* event) CV_OVERRIDE;
+    virtual void closeEvent(QCloseEvent* event) CV_OVERRIDE;
 
 private:
 
@@ -366,12 +382,13 @@ private slots:
 };
 
 
-enum type_mouse_event { mouse_up = 0, mouse_down = 1, mouse_dbclick = 2, mouse_move = 3 };
+enum type_mouse_event { mouse_up = 0, mouse_down = 1, mouse_dbclick = 2, mouse_move = 3, mouse_wheel = 4 };
 static const int tableMouseButtons[][3]={
     {CV_EVENT_LBUTTONUP, CV_EVENT_RBUTTONUP, CV_EVENT_MBUTTONUP},               //mouse_up
     {CV_EVENT_LBUTTONDOWN, CV_EVENT_RBUTTONDOWN, CV_EVENT_MBUTTONDOWN},         //mouse_down
     {CV_EVENT_LBUTTONDBLCLK, CV_EVENT_RBUTTONDBLCLK, CV_EVENT_MBUTTONDBLCLK},   //mouse_dbclick
-    {CV_EVENT_MOUSEMOVE, CV_EVENT_MOUSEMOVE, CV_EVENT_MOUSEMOVE}                //mouse_move
+    {CV_EVENT_MOUSEMOVE, CV_EVENT_MOUSEMOVE, CV_EVENT_MOUSEMOVE},               //mouse_move
+    {0, 0, 0}                                                                   //mouse_wheel, to prevent exceptions in code
 };
 
 
@@ -402,90 +419,104 @@ public:
 };
 
 
+class OCVViewPort : public ViewPort
+{
+public:
+    explicit OCVViewPort();
+    ~OCVViewPort() CV_OVERRIDE {};
+    void setMouseCallBack(CvMouseCallback callback, void* param) CV_OVERRIDE;
+
+protected:
+    void icvmouseEvent(QMouseEvent* event, type_mouse_event category);
+    void icvmouseHandler(QMouseEvent* event, type_mouse_event category, int& cv_event, int& flags);
+    virtual void icvmouseProcessing(QPointF pt, int cv_event, int flags);
+
+    CvMouseCallback mouseCallback;
+    void* mouseData;
+};
+
 
 #ifdef HAVE_QT_OPENGL
 
-class OpenGlViewPort : public QGLWidget, public ViewPort
+// Use QOpenGLWidget for Qt6 (QGLWidget is deprecated)
+#ifdef HAVE_QT6
+typedef QOpenGLWidget OpenCVQtWidgetBase;
+#else
+typedef QGLWidget OpenCVQtWidgetBase;
+#endif
+
+class OpenGlViewPort : public OpenCVQtWidgetBase, public OCVViewPort
 {
 public:
     explicit OpenGlViewPort(QWidget* parent);
-    ~OpenGlViewPort();
+    ~OpenGlViewPort() CV_OVERRIDE;
 
-    QWidget* getWidget();
+    QWidget* getWidget() CV_OVERRIDE;
 
-    void setMouseCallBack(CvMouseCallback callback, void* param);
+    void writeSettings(QSettings& settings) CV_OVERRIDE;
+    void readSettings(QSettings& settings) CV_OVERRIDE;
 
-    void writeSettings(QSettings& settings);
-    void readSettings(QSettings& settings);
+    double getRatio() CV_OVERRIDE;
+    void setRatio(int flags) CV_OVERRIDE;
 
-    double getRatio();
-    void setRatio(int flags);
+    void updateImage(const CvArr* arr) CV_OVERRIDE;
 
-    void updateImage(const CvArr* arr);
+    void startDisplayInfo(QString text, int delayms) CV_OVERRIDE;
 
-    void startDisplayInfo(QString text, int delayms);
+    void setOpenGlDrawCallback(CvOpenGlDrawCallback callback, void* userdata) CV_OVERRIDE;
+    void makeCurrentOpenGlContext() CV_OVERRIDE;
+    void updateGl() CV_OVERRIDE;
 
-    void setOpenGlDrawCallback(CvOpenGlDrawCallback callback, void* userdata);
-    void makeCurrentOpenGlContext();
-    void updateGl();
-
-    void setSize(QSize size_);
+    void setSize(QSize size_) CV_OVERRIDE;
 
 protected:
-    void initializeGL();
-    void resizeGL(int w, int h);
-    void paintGL();
+    void initializeGL() CV_OVERRIDE;
+    void resizeGL(int w, int h) CV_OVERRIDE;
+    void paintGL() CV_OVERRIDE;
 
-    void mouseMoveEvent(QMouseEvent* event);
-    void mousePressEvent(QMouseEvent* event);
-    void mouseReleaseEvent(QMouseEvent* event);
-    void mouseDoubleClickEvent(QMouseEvent* event);
+    void wheelEvent(QWheelEvent* event) CV_OVERRIDE;
+    void mouseMoveEvent(QMouseEvent* event) CV_OVERRIDE;
+    void mousePressEvent(QMouseEvent* event) CV_OVERRIDE;
+    void mouseReleaseEvent(QMouseEvent* event) CV_OVERRIDE;
+    void mouseDoubleClickEvent(QMouseEvent* event) CV_OVERRIDE;
 
-    QSize sizeHint() const;
+    QSize sizeHint() const CV_OVERRIDE;
 
 private:
     QSize size;
 
-    CvMouseCallback mouseCallback;
-    void* mouseData;
-
     CvOpenGlDrawCallback glDrawCallback;
     void* glDrawData;
-
-    void icvmouseHandler(QMouseEvent* event, type_mouse_event category, int& cv_event, int& flags);
-    void icvmouseProcessing(QPointF pt, int cv_event, int flags);
 };
 
 #endif // HAVE_QT_OPENGL
 
 
-class DefaultViewPort : public QGraphicsView, public ViewPort
+class DefaultViewPort : public QGraphicsView, public OCVViewPort
 {
     Q_OBJECT
 
 public:
     DefaultViewPort(CvWindow* centralWidget, int arg2);
-    ~DefaultViewPort();
+    ~DefaultViewPort() CV_OVERRIDE;
 
-    QWidget* getWidget();
+    QWidget* getWidget() CV_OVERRIDE;
 
-    void setMouseCallBack(CvMouseCallback callback, void* param);
+    void writeSettings(QSettings& settings) CV_OVERRIDE;
+    void readSettings(QSettings& settings) CV_OVERRIDE;
 
-    void writeSettings(QSettings& settings);
-    void readSettings(QSettings& settings);
+    double getRatio() CV_OVERRIDE;
+    void setRatio(int flags) CV_OVERRIDE;
 
-    double getRatio();
-    void setRatio(int flags);
+    void updateImage(const CvArr* arr) CV_OVERRIDE;
 
-    void updateImage(const CvArr* arr);
+    void startDisplayInfo(QString text, int delayms) CV_OVERRIDE;
 
-    void startDisplayInfo(QString text, int delayms);
+    void setOpenGlDrawCallback(CvOpenGlDrawCallback callback, void* userdata) CV_OVERRIDE;
+    void makeCurrentOpenGlContext() CV_OVERRIDE;
+    void updateGl() CV_OVERRIDE;
 
-    void setOpenGlDrawCallback(CvOpenGlDrawCallback callback, void* userdata);
-    void makeCurrentOpenGlContext();
-    void updateGl();
-
-    void setSize(QSize size_);
+    void setSize(QSize size_) CV_OVERRIDE;
 
 public slots:
     //reference:
@@ -503,16 +534,18 @@ public slots:
     void ZoomOut();
 
     void saveView();
+    void copy2Clipbrd();
 
 protected:
-    void contextMenuEvent(QContextMenuEvent* event);
-    void resizeEvent(QResizeEvent* event);
-    void paintEvent(QPaintEvent* paintEventInfo);
-    void wheelEvent(QWheelEvent* event);
-    void mouseMoveEvent(QMouseEvent* event);
-    void mousePressEvent(QMouseEvent* event);
-    void mouseReleaseEvent(QMouseEvent* event);
-    void mouseDoubleClickEvent(QMouseEvent* event);
+    void contextMenuEvent(QContextMenuEvent* event) CV_OVERRIDE;
+    void resizeEvent(QResizeEvent* event) CV_OVERRIDE;
+    void paintEvent(QPaintEvent* paintEventInfo) CV_OVERRIDE;
+
+    void wheelEvent(QWheelEvent* event) CV_OVERRIDE;
+    void mouseMoveEvent(QMouseEvent* event) CV_OVERRIDE;
+    void mousePressEvent(QMouseEvent* event) CV_OVERRIDE;
+    void mouseReleaseEvent(QMouseEvent* event) CV_OVERRIDE;
+    void mouseDoubleClickEvent(QMouseEvent* event) CV_OVERRIDE;
 
 private:
     int param_keepRatio;
@@ -522,26 +555,21 @@ private:
 
     CvMat* image2Draw_mat;
     QImage image2Draw_qt;
-    QImage image2Draw_qt_resized;
     int nbChannelOriginImage;
 
-    //for mouse callback
-    CvMouseCallback on_mouse;
-    void* on_mouse_param;
-
 
     void scaleView(qreal scaleFactor, QPointF center);
     void moveView(QPointF delta);
 
-    QPoint mouseCoordinate;
+    QPoint  mouseCoordinate;
     QPointF positionGrabbing;
-    QRect  positionCorners;
+    QRect   positionCorners;
     QTransform matrixWorld_inv;
     float ratioX, ratioY;
 
     bool isSameSize(IplImage* img1,IplImage* img2);
 
-    QSize sizeHint() const;
+    QSize sizeHint() const CV_OVERRIDE;
     QPointer<CvWindow> centralWidget;
     QPointer<QTimer> timerDisplay;
     bool drawInfo;
@@ -554,8 +582,8 @@ private:
     void draw2D(QPainter *painter);
     void drawStatusBar();
     void controlImagePosition();
-    void icvmouseHandler(QMouseEvent *event, type_mouse_event category, int &cv_event, int &flags);
-    void icvmouseProcessing(QPointF pt, int cv_event, int flags);
+
+    void icvmouseProcessing(QPointF pt, int cv_event, int flags) CV_OVERRIDE;
 
 private slots:
     void stopDisplayInfo();
