@@ -41,27 +41,20 @@
 
 #include "test_precomp.hpp"
 
-#include <cmath>
-#include <vector>
-#include <iostream>
+namespace opencv_test { namespace {
 
-using namespace cv;
-
-namespace
+void __wrap_printf_func(const char* fmt, ...)
 {
-    void __wrap_printf_func(const char* fmt, ...)
-    {
-        va_list args;
-        va_start(args, fmt);
-        char buffer[256];
-        vsprintf (buffer, fmt, args);
-        cvtest::TS::ptr()->printf(cvtest::TS::SUMMARY, buffer);
-        va_end(args);
-    }
-
-    #define PRINT_TO_LOG __wrap_printf_func
+    va_list args;
+    va_start(args, fmt);
+    char buffer[256];
+    vsnprintf (buffer, sizeof(buffer), fmt, args);
+    cvtest::TS::ptr()->printf(cvtest::TS::SUMMARY, buffer);
+    va_end(args);
 }
 
+#define PRINT_TO_LOG __wrap_printf_func
+
 #define SHOW_IMAGE
 #undef SHOW_IMAGE
 
@@ -84,6 +77,7 @@ protected:
 
     virtual void run_func() = 0;
     virtual void run_reference_func() = 0;
+    virtual float get_success_error_level(int _interpolation, int _depth) const;
     virtual void validate_results() const;
     virtual void prepare_test_data_for_reference_func();
 
@@ -119,6 +113,8 @@ String CV_ImageWarpBaseTest::interpolation_to_string(int inter) const
         str = "INTER_NEAREST";
     else if (inter == INTER_LINEAR)
         str = "INTER_LINEAR";
+    else if (inter == INTER_LINEAR_EXACT)
+        str = "INTER_LINEAR_EXACT";
     else if (inter == INTER_AREA)
         str = "INTER_AREA";
     else if (inter == INTER_CUBIC)
@@ -131,7 +127,7 @@ String CV_ImageWarpBaseTest::interpolation_to_string(int inter) const
     if (inverse)
         str += " | WARP_INVERSE_MAP";
 
-    return str.empty() ? "Unsupported/Unkown interpolation type" : str;
+    return str.empty() ? "Unsupported/Unknown interpolation type" : str;
 }
 
 Size CV_ImageWarpBaseTest::randSize(RNG& rng) const
@@ -155,8 +151,6 @@ void CV_ImageWarpBaseTest::generate_test_data()
         depth = rng.uniform(0, CV_64F);
 
     int cn = rng.uniform(1, 4);
-    while (cn == 2)
-        cn = rng.uniform(1, 4);
 
     src.create(ssize, CV_MAKE_TYPE(depth, cn));
 
@@ -167,7 +161,7 @@ void CV_ImageWarpBaseTest::generate_test_data()
         for (y = 0; y < ssize.height; y += cell_size)
             for (x = 0; x < ssize.width; x += cell_size)
                 rectangle(src, Point(x, y), Point(x + std::min<int>(cell_size, ssize.width - x), y +
-                        std::min<int>(cell_size, ssize.height - y)), Scalar::all((x + y) % 2 ? 255: 0), CV_FILLED);
+                        std::min<int>(cell_size, ssize.height - y)), Scalar::all((x + y) % 2 ? 255: 0), cv::FILLED);
     }
     else
     {
@@ -179,7 +173,7 @@ void CV_ImageWarpBaseTest::generate_test_data()
     }
 
     // generating an interpolation type
-    interpolation = rng.uniform(0, CV_INTER_LANCZOS4 + 1);
+    interpolation = rng.uniform(0, cv::INTER_LANCZOS4 + 1);
 
     // generating the dst matrix structure
     double scale_x, scale_y;
@@ -234,6 +228,20 @@ void CV_ImageWarpBaseTest::run(int)
     ts->set_gtest_status();
 }
 
+float CV_ImageWarpBaseTest::get_success_error_level(int _interpolation, int) const
+{
+    if (_interpolation == INTER_CUBIC)
+        return 1.0f;
+    else if (_interpolation == INTER_LANCZOS4)
+        return 1.0f;
+    else if (_interpolation == INTER_NEAREST)
+        return 255.0f;  // FIXIT: check is not reliable for Black/White (0/255) images
+    else if (_interpolation == INTER_AREA)
+        return 2.0f;
+    else
+        return 1.0f;
+}
+
 void CV_ImageWarpBaseTest::validate_results() const
 {
     Mat _dst;
@@ -242,15 +250,7 @@ void CV_ImageWarpBaseTest::validate_results() const
     Size dsize = dst.size(), ssize = src.size();
     int cn = _dst.channels();
     dsize.width *= cn;
-    float t = 1.0f;
-    if (interpolation == INTER_CUBIC)
-        t = 1.0f;
-    else if (interpolation == INTER_LANCZOS4)
-        t = 1.0f;
-    else if (interpolation == INTER_NEAREST)
-        t = 1.0f;
-    else if (interpolation == INTER_AREA)
-        t = 2.0f;
+    float t = get_success_error_level(interpolation & INTER_MAX, dst.depth());
 
     for (int dy = 0; dy < dsize.height; ++dy)
     {
@@ -286,10 +286,10 @@ void CV_ImageWarpBaseTest::validate_results() const
 
 #ifdef SHOW_IMAGE
                 const std::string w1("OpenCV impl (run func)"), w2("Reference func"), w3("Src image"), w4("Diff");
-                namedWindow(w1, CV_WINDOW_KEEPRATIO);
-                namedWindow(w2, CV_WINDOW_KEEPRATIO);
-                namedWindow(w3, CV_WINDOW_KEEPRATIO);
-                namedWindow(w4, CV_WINDOW_KEEPRATIO);
+                namedWindow(w1, cv::WINDOW_KEEPRATIO);
+                namedWindow(w2, cv::WINDOW_KEEPRATIO);
+                namedWindow(w3, cv::WINDOW_KEEPRATIO);
+                namedWindow(w4, cv::WINDOW_KEEPRATIO);
 
                 Mat diff;
                 absdiff(reference_dst, _dst, diff);
@@ -418,11 +418,76 @@ namespace
 
 void CV_Resize_Test::generate_test_data()
 {
-    CV_ImageWarpBaseTest::generate_test_data();
+    RNG& rng = ts->get_rng();
+
+    // generating the src matrix structure
+    Size ssize = randSize(rng), dsize;
+
+    int depth = rng.uniform(0, CV_64F);
+    while (depth == CV_8S || depth == CV_32S)
+        depth = rng.uniform(0, CV_64F);
+
+    int cn = rng.uniform(1, 4);
+
+    src.create(ssize, CV_MAKE_TYPE(depth, cn));
+
+    // generating the src matrix
+    int x, y;
+    if (cvtest::randInt(rng) % 2)
+    {
+        for (y = 0; y < ssize.height; y += cell_size)
+            for (x = 0; x < ssize.width; x += cell_size)
+                rectangle(src, Point(x, y), Point(x + std::min<int>(cell_size, ssize.width - x), y +
+                        std::min<int>(cell_size, ssize.height - y)), Scalar::all((x + y) % 2 ? 255: 0), cv::FILLED);
+    }
+    else
+    {
+        src = Scalar::all(255);
+        for (y = cell_size; y < src.rows; y += cell_size)
+            line(src, Point2i(0, y), Point2i(src.cols, y), Scalar::all(0), 1);
+        for (x = cell_size; x < src.cols; x += cell_size)
+            line(src, Point2i(x, 0), Point2i(x, src.rows), Scalar::all(0), 1);
+    }
+
+    // generating an interpolation type
+    interpolation = rng.uniform(0, cv::INTER_MAX - 1);
+
+    // generating the dst matrix structure
+    if (interpolation == INTER_AREA)
+    {
+        area_fast = rng.uniform(0., 1.) > 0.5;
+        if (area_fast)
+        {
+            scale_x = rng.uniform(2, 5);
+            scale_y = rng.uniform(2, 5);
+        }
+        else
+        {
+            scale_x = rng.uniform(1.0, 3.0);
+            scale_y = rng.uniform(1.0, 3.0);
+        }
+    }
+    else
+    {
+        scale_x = rng.uniform(0.4, 4.0);
+        scale_y = rng.uniform(0.4, 4.0);
+    }
+    CV_Assert(scale_x > 0.0f && scale_y > 0.0f);
+
+    dsize.width = saturate_cast<int>((ssize.width + scale_x - 1) / scale_x);
+    dsize.height = saturate_cast<int>((ssize.height + scale_y - 1) / scale_y);
+
+    dst = Mat::zeros(dsize, src.type());
+    reference_dst = Mat::zeros(dst.size(), CV_MAKE_TYPE(CV_32F, dst.channels()));
 
     scale_x = src.cols / static_cast<double>(dst.cols);
     scale_y = src.rows / static_cast<double>(dst.rows);
 
+    if (interpolation == INTER_AREA && (scale_x < 1.0 || scale_y < 1.0))
+        interpolation = INTER_LINEAR_EXACT;
+    if (interpolation == INTER_LINEAR_EXACT && (depth == CV_32F || depth == CV_64F))
+        interpolation = INTER_LINEAR;
+
     area_fast = interpolation == INTER_AREA &&
         fabs(scale_x - cvRound(scale_x)) < FLT_EPSILON &&
         fabs(scale_y - cvRound(scale_y)) < FLT_EPSILON;
@@ -458,7 +523,7 @@ double CV_Resize_Test::getWeight(double a, double b, int x)
 void CV_Resize_Test::resize_area()
 {
     Size ssize = src.size(), dsize = reference_dst.size();
-    CV_Assert(ssize.area() > 0 && dsize.area() > 0);
+    CV_Assert(!ssize.empty() && !dsize.empty());
     int cn = src.channels();
 
     CV_Assert(scale_x >= 1.0 && scale_y >= 1.0);
@@ -508,7 +573,7 @@ void CV_Resize_Test::resize_area()
     }
 }
 
-// for interpolation type : INTER_LINEAR, INTER_LINEAR, INTER_CUBIC, INTER_LANCZOS4
+// for interpolation type : INTER_LINEAR, INTER_LINEAR_EXACT, INTER_CUBIC, INTER_LANCZOS4
 void CV_Resize_Test::resize_1d(const Mat& _src, Mat& _dst, int dy, const dim& _dim)
 {
     Size dsize = _dst.size();
@@ -528,9 +593,9 @@ void CV_Resize_Test::resize_1d(const Mat& _src, Mat& _dst, int dy, const dim& _d
                 xyD[r] = xyS[r];
         }
     }
-    else if (interpolation == INTER_LINEAR || interpolation == INTER_CUBIC || interpolation == INTER_LANCZOS4)
+    else if (interpolation == INTER_LINEAR || interpolation == INTER_LINEAR_EXACT || interpolation == INTER_CUBIC || interpolation == INTER_LANCZOS4)
     {
-        interpolate_method inter_func = inter_array[interpolation - (interpolation == INTER_LANCZOS4 ? 2 : 1)];
+        interpolate_method inter_func = inter_array[interpolation - (interpolation == INTER_LANCZOS4 ? 2 : interpolation == INTER_LINEAR_EXACT ? 5 : 1)];
         size_t elemsize = _src.elemSize();
 
         int ofs = 0, ksize = 2;
@@ -585,7 +650,7 @@ void CV_Resize_Test::generate_buffer(double scale, dim& _dim)
 void CV_Resize_Test::resize_generic()
 {
     Size dsize = reference_dst.size(), ssize = src.size();
-    CV_Assert(dsize.area() > 0 && ssize.area() > 0);
+    CV_Assert(!dsize.empty() && !ssize.empty());
 
     dim dims[] = { dim(dsize.width), dim(dsize.height) };
     if (interpolation == INTER_NEAREST)
@@ -605,12 +670,15 @@ void CV_Resize_Test::resize_generic()
     for (int dy = 0; dy < tmp.rows; ++dy)
         resize_1d(src, tmp, dy, dims[0]);
 
-    transpose(tmp, tmp);
-    transpose(reference_dst, reference_dst);
+    cv::Mat tmp_t(tmp.cols, tmp.rows, tmp.type());
+    cvtest::transpose(tmp, tmp_t);
+    cv::Mat reference_dst_t(reference_dst.cols, reference_dst.rows, reference_dst.type());
+    cvtest::transpose(reference_dst, reference_dst_t);
 
-    for (int dy = 0; dy < tmp.rows; ++dy)
-        resize_1d(tmp, reference_dst, dy, dims[1]);
-    transpose(reference_dst, reference_dst);
+    for (int dy = 0; dy < tmp_t.rows; ++dy)
+        resize_1d(tmp_t, reference_dst_t, dy, dims[1]);
+
+    cvtest::transpose(reference_dst_t, reference_dst);
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -786,7 +854,7 @@ const char* CV_Remap_Test::borderType_to_string() const
         return "BORDER_WRAP";
     if (borderType == BORDER_REFLECT_101)
         return "BORDER_REFLECT_101";
-    return "Unsupported/Unkown border type";
+    return "Unsupported/Unknown border type";
 }
 
 void CV_Remap_Test::prepare_test_data_for_reference_func()
@@ -812,7 +880,7 @@ void CV_Remap_Test::remap_nearest(const Mat& _src, Mat& _dst)
     CV_Assert(mapx.type() == CV_16SC2 && mapy.empty());
 
     Size ssize = _src.size(), dsize = _dst.size();
-    CV_Assert(ssize.area() > 0 && dsize.area() > 0);
+    CV_Assert(!ssize.empty() && !dsize.empty());
     int cn = _src.channels();
 
     for (int dy = 0; dy < dsize.height; ++dy)
@@ -863,7 +931,7 @@ void CV_Remap_Test::remap_generic(const Mat& _src, Mat& _dst)
     else if (interpolation == INTER_LANCZOS4)
         ksize = 8;
     else if (interpolation != INTER_LINEAR)
-        assert(0);
+        CV_Assert(0);
     int ofs = (ksize / 2) - 1;
 
     CV_Assert(_src.depth() == CV_32F && _dst.type() == _src.type());
@@ -969,7 +1037,7 @@ public:
 
 protected:
     virtual void generate_test_data();
-    virtual void prepare_test_data_for_reference_func();
+    virtual float get_success_error_level(int _interpolation, int _depth) const;
 
     virtual void run_func();
     virtual void run_reference_func();
@@ -1010,7 +1078,7 @@ void CV_WarpAffine_Test::generate_test_data()
 
     // warp_matrix is inverse
     if (rng.uniform(0., 1.) > 0)
-        interpolation |= CV_WARP_INVERSE_MAP;
+        interpolation |= cv::WARP_INVERSE_MAP;
 }
 
 void CV_WarpAffine_Test::run_func()
@@ -1018,24 +1086,24 @@ void CV_WarpAffine_Test::run_func()
     cv::warpAffine(src, dst, M, dst.size(), interpolation, borderType, borderValue);
 }
 
-void CV_WarpAffine_Test::prepare_test_data_for_reference_func()
+float CV_WarpAffine_Test::get_success_error_level(int _interpolation, int _depth) const
 {
-    CV_ImageWarpBaseTest::prepare_test_data_for_reference_func();
+    return _depth == CV_8U ? 0 : CV_ImageWarpBaseTest::get_success_error_level(_interpolation, _depth);
 }
 
 void CV_WarpAffine_Test::run_reference_func()
 {
-    prepare_test_data_for_reference_func();
-
-    warpAffine(src, reference_dst);
+    Mat tmp = Mat::zeros(dst.size(), dst.type());
+    warpAffine(src, tmp);
+    tmp.convertTo(reference_dst, reference_dst.depth());
 }
 
 void CV_WarpAffine_Test::warpAffine(const Mat& _src, Mat& _dst)
 {
     Size dsize = _dst.size();
 
-    CV_Assert(_src.size().area() > 0);
-    CV_Assert(dsize.area() > 0);
+    CV_Assert(!_src.empty());
+    CV_Assert(!dsize.empty());
     CV_Assert(_src.type() == _dst.type());
 
     Mat tM;
@@ -1051,14 +1119,14 @@ void CV_WarpAffine_Test::warpAffine(const Mat& _src, Mat& _dst)
     else
         mapy = Mat();
 
-    if (!(interpolation & CV_WARP_INVERSE_MAP))
+    if (!(interpolation & cv::WARP_INVERSE_MAP))
         invertAffineTransform(tM.clone(), tM);
 
     const int AB_BITS = MAX(10, (int)INTER_BITS);
     const int AB_SCALE = 1 << AB_BITS;
     int round_delta = (inter == INTER_NEAREST) ? AB_SCALE / 2 : (AB_SCALE / INTER_TAB_SIZE / 2);
 
-    const double* data_tM = tM.ptr<double>(0);
+    const softdouble* data_tM = tM.ptr<softdouble>(0);
     for (int dy = 0; dy < dsize.height; ++dy)
     {
         short* yM = mapx.ptr<short>(dy);
@@ -1097,6 +1165,7 @@ public:
 
 protected:
     virtual void generate_test_data();
+    virtual float get_success_error_level(int _interpolation, int _depth) const;
 
     virtual void run_func();
     virtual void run_reference_func();
@@ -1139,19 +1208,24 @@ void CV_WarpPerspective_Test::run_func()
     cv::warpPerspective(src, dst, M, dst.size(), interpolation, borderType, borderValue);
 }
 
+float CV_WarpPerspective_Test::get_success_error_level(int _interpolation, int _depth) const
+{
+    return CV_ImageWarpBaseTest::get_success_error_level(_interpolation, _depth);
+}
+
 void CV_WarpPerspective_Test::run_reference_func()
 {
-    prepare_test_data_for_reference_func();
-
-    warpPerspective(src, reference_dst);
+    Mat tmp = Mat::zeros(dst.size(), dst.type());
+    warpPerspective(src, tmp);
+    tmp.convertTo(reference_dst, reference_dst.depth());
 }
 
 void CV_WarpPerspective_Test::warpPerspective(const Mat& _src, Mat& _dst)
 {
     Size ssize = _src.size(), dsize = _dst.size();
 
-    CV_Assert(ssize.area() > 0);
-    CV_Assert(dsize.area() > 0);
+    CV_Assert(!ssize.empty());
+    CV_Assert(!dsize.empty());
     CV_Assert(_src.type() == _dst.type());
 
     if (M.depth() != CV_64F)
@@ -1161,7 +1235,7 @@ void CV_WarpPerspective_Test::warpPerspective(const Mat& _src, Mat& _dst)
         M = tmp;
     }
 
-    if (!(interpolation & CV_WARP_INVERSE_MAP))
+    if (!(interpolation & cv::WARP_INVERSE_MAP))
     {
         Mat tmp;
         invert(M, tmp);
@@ -1195,7 +1269,7 @@ void CV_WarpPerspective_Test::warpPerspective(const Mat& _src, Mat& _dst)
                 continue;
             }
 
-            den *= INTER_TAB_SIZE;
+            den *= static_cast<double>(INTER_TAB_SIZE);
             int v0 = saturate_cast<int>((tM[0] * dx + tM[1] * dy + tM[2]) * den);
             int v1 = saturate_cast<int>((tM[3] * dx + tM[4] * dy + tM[5]) * den);
 
@@ -1223,7 +1297,7 @@ TEST(Imgproc_WarpPerspective_Test, accuracy) { CV_WarpPerspective_Test test; tes
 
 #ifdef OPENCV_TEST_BIGDATA
 
-CV_ENUM(Interpolation, INTER_NEAREST, INTER_LINEAR, INTER_CUBIC, INTER_AREA)
+CV_ENUM(Interpolation, INTER_NEAREST, INTER_LINEAR, INTER_LINEAR_EXACT, INTER_CUBIC, INTER_AREA)
 
 class Imgproc_Resize :
         public ::testing::TestWithParam<Interpolation>
@@ -1249,3 +1323,5 @@ TEST_P(Imgproc_Resize, BigSize)
 INSTANTIATE_TEST_CASE_P(Imgproc, Imgproc_Resize, Interpolation::all());
 
 #endif
+
+}} // namespace
