@@ -44,26 +44,30 @@
 //
 
 #include "precomp.hpp"
+#include "cap_interface.hpp"
+
+using namespace cv;
 
 #ifdef HAVE_ARAVIS_API
 
 #include <arv.h>
 
 //
-// This file provides wrapper for using Aravis SDK library to access GigE Vision cameras.
-// Aravis library (version 0.4 or 0.6) shall be installed else this code will not be included in build.
+// This file provides wrapper for using Aravis SDK library to access GigE and USB 3 Vision cameras.
+// Aravis library (version 0.8) shall be installed else this code will not be included in build.
 //
 // To include this module invoke cmake with -DWITH_ARAVIS=ON
 //
 // Please obvserve, that jumbo frames are required when high fps & 16bit data is selected.
 // (camera, switches/routers and the computer this software is running on)
 //
-// Basic usage: VideoCapture cap(CAP_ARAVIS + <camera id>);
+// Basic usage: VideoCapture cap(<camera id>, CAP_ARAVIS);
 //
 // Supported properties:
 //  read/write
 //      CAP_PROP_AUTO_EXPOSURE(0|1)
 //      CAP_PROP_EXPOSURE(t), t in seconds
+//      CAP_PROP_BRIGHTNESS (ev), exposure compensation in EV for auto exposure algorithm
 //      CAP_PROP_GAIN(g), g >=0 or -1 for automatic control if CAP_PROP_AUTO_EXPOSURE is true
 //      CAP_PROP_FPS(f)
 //      CAP_PROP_FOURCC(type)
@@ -77,35 +81,40 @@
 //      video/x-raw, fourcc:'GREY'  -> 8bit, 1 channel
 //      video/x-raw, fourcc:'Y800'  -> 8bit, 1 channel
 //      video/x-raw, fourcc:'Y12 '  -> 12bit, 1 channel
+//      video/x-raw, fourcc:'Y16 '  -> 16bit, 1 channel
+//      video/x-raw, fourcc:'GRBG'  -> 8bit, 1 channel
 //
 
 #define MODE_GREY   CV_FOURCC_MACRO('G','R','E','Y')
 #define MODE_Y800   CV_FOURCC_MACRO('Y','8','0','0')
 #define MODE_Y12    CV_FOURCC_MACRO('Y','1','2',' ')
+#define MODE_Y16    CV_FOURCC_MACRO('Y','1','6',' ')
+#define MODE_GRBG   CV_FOURCC_MACRO('G','R','B','G')
 
 #define CLIP(a,b,c) (cv::max(cv::min((a),(c)),(b)))
 
 /********************* Capturing video from camera via Aravis *********************/
 
-class CvCaptureCAM_Aravis : public CvCapture
+class CvCaptureCAM_Aravis : public IVideoCapture
 {
 public:
     CvCaptureCAM_Aravis();
-    virtual ~CvCaptureCAM_Aravis()
+    ~CvCaptureCAM_Aravis()
     {
         close();
     }
 
-    virtual bool open(int);
-    virtual void close();
-    virtual double getProperty(int) const;
-    virtual bool setProperty(int, double);
-    virtual bool grabFrame();
-    virtual IplImage* retrieveFrame(int);
-    virtual int getCaptureDomain()
+    bool open(int);
+    void close();
+    double getProperty(int) const CV_OVERRIDE;
+    bool setProperty(int, double) CV_OVERRIDE;
+    bool grabFrame() CV_OVERRIDE;
+    bool retrieveFrame(int, OutputArray) CV_OVERRIDE;
+    int getCaptureDomain() CV_OVERRIDE
     {
-        return CV_CAP_ARAVIS;
+        return cv::CAP_ARAVIS;
     }
+    bool isOpened() const CV_OVERRIDE { return stream != NULL; }
 
 protected:
     bool create(int);
@@ -116,7 +125,7 @@ protected:
 
     bool getDeviceNameById(int id, std::string &device);
 
-    void autoExposureControl(IplImage*);
+    void autoExposureControl(const Mat &);
 
     ArvCamera       *camera;                // Camera to control.
     ArvStream       *stream;                // Object for video stream reception.
@@ -124,12 +133,12 @@ protected:
 
     unsigned int    payload;                // Width x height x Pixel width.
 
-    int             widthMin;               // Camera sensor minium width.
+    int             widthMin;               // Camera sensor minimum width.
     int             widthMax;               // Camera sensor maximum width.
-    int             heightMin;              // Camera sensor minium height.
+    int             heightMin;              // Camera sensor minimum height.
     int             heightMax;              // Camera sensor maximum height.
     bool            fpsAvailable;
-    double          fpsMin;                 // Camera minium fps.
+    double          fpsMin;                 // Camera minimum fps.
     double          fpsMax;                 // Camera maximum fps.
     bool            gainAvailable;
     double          gainMin;                // Camera minimum gain.
@@ -139,12 +148,11 @@ protected:
     double          exposureMax;            // Camera's maximum exposure time.
 
     bool            controlExposure;        // Flag if automatic exposure shall be done by this SW
+    double          exposureCompensation;
     bool            autoGain;
     double          targetGrey;             // Target grey value (mid grey))
-
-    gint64          *pixelFormats;
-    guint           pixelFormatsCnt;
-
+    bool            softwareTriggered;      // Flag if the camera is software triggered
+    bool            allowAutoTrigger;       // Flag that user allowed to trigger software triggered cameras automatically
 
     int             num_buffers;            // number of payload transmission buffers
 
@@ -162,8 +170,6 @@ protected:
 
     unsigned        frameID;                // current frame id
     unsigned        prevFrameID;
-
-    IplImage        *frame;                 // local frame copy
 };
 
 
@@ -179,11 +185,12 @@ CvCaptureCAM_Aravis::CvCaptureCAM_Aravis()
     xoffset = yoffset = width = height = 0;
     fpsMin = fpsMax = gainMin = gainMax = exposureMin = exposureMax = 0;
     controlExposure = false;
+    exposureCompensation = 0;
     targetGrey = 0;
     frameID = prevFrameID = 0;
+    allowAutoTrigger = false;
 
-    num_buffers = 50;
-    frame = NULL;
+    num_buffers = 10;
 }
 
 void CvCaptureCAM_Aravis::close()
@@ -214,7 +221,7 @@ bool CvCaptureCAM_Aravis::create( int index )
     if(!getDeviceNameById(index, deviceName))
         return false;
 
-    return NULL != (camera = arv_camera_new(deviceName.c_str()));
+    return NULL != (camera = arv_camera_new(deviceName.c_str(), NULL));
 }
 
 bool CvCaptureCAM_Aravis::init_buffers()
@@ -223,17 +230,18 @@ bool CvCaptureCAM_Aravis::init_buffers()
         g_object_unref(stream);
         stream = NULL;
     }
-    if( (stream = arv_camera_create_stream(camera, NULL, NULL)) ) {
-        g_object_set(stream,
-            "socket-buffer", ARV_GV_STREAM_SOCKET_BUFFER_AUTO,
-            "socket-buffer-size", 0, NULL);
-        g_object_set(stream,
-            "packet-resend", ARV_GV_STREAM_PACKET_RESEND_NEVER, NULL);
-        g_object_set(stream,
-            "packet-timeout", (unsigned) 40000,
-            "frame-retention", (unsigned) 200000, NULL);
-
-        payload = arv_camera_get_payload (camera);
+    if( (stream = arv_camera_create_stream(camera, NULL, NULL, NULL)) ) {
+        if( arv_camera_is_gv_device(camera) ) {
+            g_object_set(stream,
+                "socket-buffer", ARV_GV_STREAM_SOCKET_BUFFER_AUTO,
+                "socket-buffer-size", 0, NULL);
+            g_object_set(stream,
+                "packet-resend", ARV_GV_STREAM_PACKET_RESEND_NEVER, NULL);
+            g_object_set(stream,
+                "packet-timeout", (unsigned) 40000,
+                "frame-retention", (unsigned) 200000, NULL);
+        }
+        payload = arv_camera_get_payload (camera, NULL);
 
         for (int i = 0; i < num_buffers; i++)
             arv_stream_push_buffer(stream, arv_buffer_new(payload, NULL));
@@ -248,24 +256,23 @@ bool CvCaptureCAM_Aravis::open( int index )
 {
     if(create(index)) {
         // fetch properties bounds
-        pixelFormats = arv_camera_get_available_pixel_formats(camera, &pixelFormatsCnt);
+        arv_camera_get_width_bounds(camera, &widthMin, &widthMax, NULL);
+        arv_camera_get_height_bounds(camera, &heightMin, &heightMax, NULL);
+        arv_camera_set_region(camera, 0, 0, widthMax, heightMax, NULL);
 
-        arv_camera_get_width_bounds(camera, &widthMin, &widthMax);
-        arv_camera_get_height_bounds(camera, &heightMin, &heightMax);
-        arv_camera_set_region(camera, 0, 0, widthMax, heightMax);
-
-        if( (fpsAvailable = arv_camera_is_frame_rate_available(camera)) )
-            arv_camera_get_frame_rate_bounds(camera, &fpsMin, &fpsMax);
-        if( (gainAvailable = arv_camera_is_gain_available(camera)) )
-            arv_camera_get_gain_bounds (camera, &gainMin, &gainMax);
-        if( (exposureAvailable = arv_camera_is_exposure_time_available(camera)) )
-            arv_camera_get_exposure_time_bounds (camera, &exposureMin, &exposureMax);
+        if( (fpsAvailable = arv_camera_is_frame_rate_available(camera, NULL)) )
+            arv_camera_get_frame_rate_bounds(camera, &fpsMin, &fpsMax, NULL);
+        if( (gainAvailable = arv_camera_is_gain_available(camera, NULL)) )
+            arv_camera_get_gain_bounds (camera, &gainMin, &gainMax, NULL);
+        if( (exposureAvailable = arv_camera_is_exposure_time_available(camera, NULL)) )
+            arv_camera_get_exposure_time_bounds (camera, &exposureMin, &exposureMax, NULL);
 
         // get initial values
-        pixelFormat = arv_camera_get_pixel_format(camera);
-        exposure = exposureAvailable ? arv_camera_get_exposure_time(camera) : 0;
-        gain = gainAvailable ? arv_camera_get_gain(camera) : 0;
-        fps = arv_camera_get_frame_rate(camera);
+        pixelFormat = arv_camera_get_pixel_format(camera, NULL);
+        exposure = exposureAvailable ? arv_camera_get_exposure_time(camera, NULL) : 0;
+        gain = gainAvailable ? arv_camera_get_gain(camera, NULL) : 0;
+        fps = arv_camera_get_frame_rate(camera, NULL);
+        softwareTriggered = (strcmp(arv_camera_get_trigger_source(camera, NULL), "Software") == 0);
 
         return startCapture();
     }
@@ -281,6 +288,9 @@ bool CvCaptureCAM_Aravis::grabFrame()
         ArvBuffer *arv_buffer = NULL;
         int max_tries = 10;
         int tries = 0;
+        if (softwareTriggered && allowAutoTrigger) {
+            arv_camera_software_trigger (camera, NULL);
+        }
         for(; tries < max_tries; tries ++) {
             arv_buffer = arv_stream_timeout_pop_buffer (stream, 200000);
             if (arv_buffer != NULL && arv_buffer_get_status (arv_buffer) != ARV_BUFFER_STATUS_SUCCESS) {
@@ -291,10 +301,10 @@ bool CvCaptureCAM_Aravis::grabFrame()
             size_t buffer_size;
             framebuffer = (void*)arv_buffer_get_data (arv_buffer, &buffer_size);
 
-            // retieve image size properites
+            // retrieve image size properties
             arv_buffer_get_image_region (arv_buffer, &xoffset, &yoffset, &width, &height);
 
-            // retieve image ID set by camera
+            // retrieve image ID set by camera
             frameID = arv_buffer_get_frame_id(arv_buffer);
 
             arv_stream_push_buffer(stream, arv_buffer);
@@ -304,49 +314,37 @@ bool CvCaptureCAM_Aravis::grabFrame()
     return false;
 }
 
-IplImage* CvCaptureCAM_Aravis::retrieveFrame(int)
+bool CvCaptureCAM_Aravis::retrieveFrame(int, OutputArray arr)
 {
     if(framebuffer) {
         int depth = 0, channels = 0;
         switch(pixelFormat) {
             case ARV_PIXEL_FORMAT_MONO_8:
-                depth = IPL_DEPTH_8U;
+            case ARV_PIXEL_FORMAT_BAYER_GR_8:
+                depth = CV_8U;
                 channels = 1;
                 break;
             case ARV_PIXEL_FORMAT_MONO_12:
-                depth = IPL_DEPTH_16U;
+            case ARV_PIXEL_FORMAT_MONO_16:
+                depth = CV_16U;
                 channels = 1;
                 break;
+            default:
+                return false;
         }
-        if(depth && channels) {
-            IplImage src;
-            cvInitImageHeader( &src, cvSize( width, height ), depth, channels, IPL_ORIGIN_TL, 4 );
-
-            cvSetData( &src, framebuffer, src.widthStep );
-            if( !frame ||
-                 frame->width != src.width ||
-                 frame->height != src.height ||
-                 frame->depth != src.depth ||
-                 frame->nChannels != src.nChannels) {
-
-                cvReleaseImage( &frame );
-                frame = cvCreateImage( cvGetSize(&src), src.depth, channels );
-            }
-            cvCopy(&src, frame);
-
-            if(controlExposure && ((frameID - prevFrameID) > 1)) {
-                // control exposure every second frame
-                // i.e. skip frame taken with previous exposure setup
-                autoExposureControl(frame);
-            }
-
-            return frame;
+        Mat src(Size( width, height ), CV_MAKE_TYPE(depth, channels), framebuffer);
+        if(controlExposure && ((frameID - prevFrameID) >= 3)) {
+            // control exposure every third frame
+            // i.e. skip frame taken with previous exposure setup
+            autoExposureControl(src);
         }
+        src.copyTo(arr);
+        return true;
     }
-    return NULL;
+    return false;
 }
 
-void CvCaptureCAM_Aravis::autoExposureControl(IplImage* image)
+void CvCaptureCAM_Aravis::autoExposureControl(const Mat & image)
 {
     // Software control of exposure parameters utilizing
     // automatic change of exposure time & gain
@@ -355,10 +353,8 @@ void CvCaptureCAM_Aravis::autoExposureControl(IplImage* image)
     // - to increase brightness, first increase time then gain
     // - to decrease brightness, first decrease gain then time
 
-    cv::Mat m = cv::cvarrToMat(image);
-
     // calc mean value for luminance or green channel
-    double brightness = cv::mean(m)[image->nChannels > 1 ? 1 : 0];
+    double brightness = cv::mean(image)[image.channels() > 1 ? 1 : 0];
     if(brightness < 1) brightness = 1;
 
     // mid point - 100 % means no change
@@ -372,27 +368,28 @@ void CvCaptureCAM_Aravis::autoExposureControl(IplImage* image)
     midGrey = brightness;
 
     double maxe = 1e6 / fps;
-    double ne = CLIP( ( exposure * d ) / dmid, exposureMin, maxe);
+    double ne = CLIP( ( exposure * d ) / ( dmid * pow(sqrt(2), -2 * exposureCompensation) ), exposureMin, maxe);
 
     // if change of value requires intervention
-    if(fabs(d-dmid) > 5) {
+    if(std::fabs(d-dmid) > 5) {
         double ev, ng = 0;
 
         if(gainAvailable && autoGain) {
             ev = log( d / dmid ) / log(2);
-            ng = CLIP( gain + ev, gainMin, gainMax);
+            ng = CLIP( gain + ev + exposureCompensation, gainMin, gainMax);
 
             if( ng < gain ) {
-                // piority 1 - reduce gain
-                arv_camera_set_gain(camera, (gain = ng));
+                // priority 1 - reduce gain
+                arv_camera_set_gain(camera, (gain = ng), NULL);
                 return;
             }
         }
 
         if(exposureAvailable) {
-            if(abs(exposure - ne) > 2) {
-                // priority 2 - control of exposure time
-                arv_camera_set_exposure_time(camera, (exposure = ne) );
+            // priority 2 - control of exposure time
+            if(std::fabs(exposure - ne) > 2) {
+                // we have not yet reach the max-e level
+                arv_camera_set_exposure_time(camera, (exposure = ne), NULL);
                 return;
             }
         }
@@ -401,12 +398,12 @@ void CvCaptureCAM_Aravis::autoExposureControl(IplImage* image)
             if(exposureAvailable) {
                 // exposure at maximum - increase gain if possible
                 if(ng > gain && ng < gainMax && ne >= maxe) {
-                    arv_camera_set_gain(camera, (gain = ng));
+                    arv_camera_set_gain(camera, (gain = ng), NULL);
                     return;
                 }
             } else {
                 // priority 3 - increase gain
-                arv_camera_set_gain(camera, (gain = ng));
+                arv_camera_set_gain(camera, (gain = ng), NULL);
                 return;
             }
         }
@@ -416,7 +413,7 @@ void CvCaptureCAM_Aravis::autoExposureControl(IplImage* image)
     if(gainAvailable && autoGain && exposureAvailable) {
         if(gain > gainMin && exposure < maxe) {
             exposure = CLIP( ne * 1.05, exposureMin, maxe);
-            arv_camera_set_exposure_time(camera, exposure );
+            arv_camera_set_exposure_time(camera, exposure, NULL);
         }
     }
 }
@@ -424,50 +421,57 @@ void CvCaptureCAM_Aravis::autoExposureControl(IplImage* image)
 double CvCaptureCAM_Aravis::getProperty( int property_id ) const
 {
     switch(property_id) {
-        case CV_CAP_PROP_POS_MSEC:
+        case CAP_PROP_POS_MSEC:
             return (double)frameID/fps;
 
-        case CV_CAP_PROP_FRAME_WIDTH:
+        case CAP_PROP_FRAME_WIDTH:
             return width;
 
-        case CV_CAP_PROP_FRAME_HEIGHT:
+        case CAP_PROP_FRAME_HEIGHT:
             return height;
 
-        case CV_CAP_PROP_AUTO_EXPOSURE:
+        case CAP_PROP_AUTO_EXPOSURE:
             return (controlExposure ? 1 : 0);
 
-        case CV_CAP_PROP_EXPOSURE:
+    case CAP_PROP_BRIGHTNESS:
+        return exposureCompensation;
+
+        case CAP_PROP_EXPOSURE:
             if(exposureAvailable) {
                 /* exposure time in seconds, like 1/100 s */
-                return arv_camera_get_exposure_time(camera) / 1e6;
+                return arv_camera_get_exposure_time(camera, NULL) / 1e6;
             }
             break;
 
-        case CV_CAP_PROP_FPS:
+        case CAP_PROP_FPS:
             if(fpsAvailable) {
-                return arv_camera_get_frame_rate(camera);
+                return arv_camera_get_frame_rate(camera, NULL);
             }
             break;
 
-        case CV_CAP_PROP_GAIN:
+        case CAP_PROP_GAIN:
             if(gainAvailable) {
-                return arv_camera_get_gain(camera);
+                return arv_camera_get_gain(camera, NULL);
             }
             break;
 
-        case CV_CAP_PROP_FOURCC:
+        case CAP_PROP_FOURCC:
             {
-                ArvPixelFormat currFormat = arv_camera_get_pixel_format(camera);
+                ArvPixelFormat currFormat = arv_camera_get_pixel_format(camera, NULL);
                 switch( currFormat ) {
                     case ARV_PIXEL_FORMAT_MONO_8:
                         return MODE_Y800;
                     case ARV_PIXEL_FORMAT_MONO_12:
                         return MODE_Y12;
+                    case ARV_PIXEL_FORMAT_MONO_16:
+                        return MODE_Y16;
+                    case ARV_PIXEL_FORMAT_BAYER_GR_8:
+                        return MODE_GRBG;
                 }
             }
             break;
 
-        case CV_CAP_PROP_BUFFERSIZE:
+        case CAP_PROP_BUFFERSIZE:
             if(stream) {
                 int in, out;
                 arv_stream_get_n_buffers(stream, &in, &out);
@@ -475,6 +479,12 @@ double CvCaptureCAM_Aravis::getProperty( int property_id ) const
                 return out;
             }
             break;
+
+        case cv::CAP_PROP_ARAVIS_AUTOTRIGGER:
+        {
+            return allowAutoTrigger ? 1. : 0.;
+        }
+        break;
     }
     return -1.0;
 }
@@ -482,40 +492,43 @@ double CvCaptureCAM_Aravis::getProperty( int property_id ) const
 bool CvCaptureCAM_Aravis::setProperty( int property_id, double value )
 {
     switch(property_id) {
-        case CV_CAP_PROP_AUTO_EXPOSURE:
+        case CAP_PROP_AUTO_EXPOSURE:
             if(exposureAvailable || gainAvailable) {
                 if( (controlExposure = (bool)(int)value) ) {
-                    exposure = exposureAvailable ? arv_camera_get_exposure_time(camera) : 0;
-                    gain = gainAvailable ? arv_camera_get_gain(camera) : 0;
+                    exposure = exposureAvailable ? arv_camera_get_exposure_time(camera, NULL) : 0;
+                    gain = gainAvailable ? arv_camera_get_gain(camera, NULL) : 0;
                 }
             }
             break;
+    case CAP_PROP_BRIGHTNESS:
+       exposureCompensation = CLIP(value, -3., 3.);
+       break;
 
-        case CV_CAP_PROP_EXPOSURE:
+        case CAP_PROP_EXPOSURE:
             if(exposureAvailable) {
                 /* exposure time in seconds, like 1/100 s */
                 value *= 1e6; // -> from s to us
 
-                arv_camera_set_exposure_time(camera, exposure = CLIP(value, exposureMin, exposureMax));
+                arv_camera_set_exposure_time(camera, exposure = CLIP(value, exposureMin, exposureMax), NULL);
                 break;
             } else return false;
 
-        case CV_CAP_PROP_FPS:
+        case CAP_PROP_FPS:
             if(fpsAvailable) {
-                arv_camera_set_frame_rate(camera, fps = CLIP(value, fpsMin, fpsMax));
+                arv_camera_set_frame_rate(camera, fps = CLIP(value, fpsMin, fpsMax), NULL);
                 break;
             } else return false;
 
-        case CV_CAP_PROP_GAIN:
+        case CAP_PROP_GAIN:
             if(gainAvailable) {
                 if ( (autoGain = (-1 == value) ) )
                     break;
 
-                arv_camera_set_gain(camera, gain = CLIP(value, gainMin, gainMax));
+                arv_camera_set_gain(camera, gain = CLIP(value, gainMin, gainMax), NULL);
                 break;
             } else return false;
 
-        case CV_CAP_PROP_FOURCC:
+        case CAP_PROP_FOURCC:
             {
                 ArvPixelFormat newFormat = pixelFormat;
                 switch((int)value) {
@@ -528,16 +541,24 @@ bool CvCaptureCAM_Aravis::setProperty( int property_id, double value )
                         newFormat = ARV_PIXEL_FORMAT_MONO_12;
                         targetGrey = 2048;
                         break;
+                    case MODE_Y16:
+                        newFormat = ARV_PIXEL_FORMAT_MONO_16;
+                        targetGrey = 32768;
+                        break;
+                    case MODE_GRBG:
+                        newFormat = ARV_PIXEL_FORMAT_BAYER_GR_8;
+                        targetGrey = 128;
+                        break;
                 }
                 if(newFormat != pixelFormat) {
                     stopCapture();
-                    arv_camera_set_pixel_format(camera, pixelFormat = newFormat);
+                    arv_camera_set_pixel_format(camera, pixelFormat = newFormat, NULL);
                     startCapture();
                 }
             }
             break;
 
-        case CV_CAP_PROP_BUFFERSIZE:
+        case CAP_PROP_BUFFERSIZE:
             {
                 int x = (int)value;
                 if((x > 0) && (x != num_buffers)) {
@@ -548,6 +569,11 @@ bool CvCaptureCAM_Aravis::setProperty( int property_id, double value )
             }
             break;
 
+        case cv::CAP_PROP_ARAVIS_AUTOTRIGGER:
+            {
+                allowAutoTrigger = (bool) value;
+            }
+            break;
 
         default:
             return false;
@@ -558,7 +584,7 @@ bool CvCaptureCAM_Aravis::setProperty( int property_id, double value )
 
 void CvCaptureCAM_Aravis::stopCapture()
 {
-    arv_camera_stop_acquisition(camera);
+    arv_camera_stop_acquisition(camera, NULL);
 
     if(stream) {
         g_object_unref(stream);
@@ -569,24 +595,20 @@ void CvCaptureCAM_Aravis::stopCapture()
 bool CvCaptureCAM_Aravis::startCapture()
 {
     if(init_buffers() ) {
-        arv_camera_set_acquisition_mode(camera, ARV_ACQUISITION_MODE_CONTINUOUS);
-        arv_device_set_string_feature_value(arv_camera_get_device (camera), "TriggerMode" , "Off");
-        arv_camera_start_acquisition(camera);
+        arv_camera_set_acquisition_mode(camera, ARV_ACQUISITION_MODE_CONTINUOUS, NULL);
+        arv_camera_start_acquisition(camera, NULL);
 
         return true;
     }
     return false;
 }
 
-CvCapture* cvCreateCameraCapture_Aravis( int index )
+cv::Ptr<cv::IVideoCapture> cv::create_Aravis_capture( int index )
 {
-    CvCaptureCAM_Aravis* capture = new CvCaptureCAM_Aravis;
-
+    Ptr<CvCaptureCAM_Aravis> capture = makePtr<CvCaptureCAM_Aravis>();
     if(capture->open(index)) {
         return capture;
     }
-
-    delete capture;
     return NULL;
 }
 #endif
